#!/bin/bash
LC_NUMERIC=C
dirname=plots
mode=${1:-"x11"}
plot_id=1

if [ $mode == "eps" ] ; then
	lw=3
else
	lw=1
fi

# create files (loaded by gnuplot) containing the relative positions
# of the lables (numbers) written on top of the bars
function create_label_positions()
{
    case "$1" in
	3)
	    label_y_offset=`echo "$max_y/100 * 3" | bc -l`
	    perl -ane 'if ( "$F[0]" ne "#" ) { print "set label \"$F[2]\" at $F[0]-.47,$F[2]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_1.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) { print "set label \"$F[3]\" at $F[0]-.12,$F[3]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_2.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) { print "set label \"$F[4]\" at $F[0]+.15,$F[4]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_3.plt
    ;;
	4)
	    label_y_offset=`echo "$max_y/100 * 3" | bc -l`

	    perl -ane 'if ( "$F[0]" ne "#" ) { print "set label \"$F[2]\" at $F[0]-.47,$F[2]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_1.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[3]\" at $F[0]-.21,$F[3]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_2.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[4]\" at $F[0]+.01,$F[4]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_3.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[5]\" at $F[0]+.21,$F[5]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_4.plt
	    ;;
	5)
	    label_y_offset=`echo "$max_y/100 * 3" | bc -l`

	    perl -ane 'if ( "$F[0]" ne "#" ) { print "set label \"$F[2]\" at $F[0]-.47,$F[2]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_1.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[3]\" at $F[0]-.22,$F[3]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_2.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[4]\" at $F[0]-.06,$F[4]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_3.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[5]\" at $F[0]+.13,$F[5]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_4.plt
	    perl -ane 'if ( "$F[0]" ne "#" ) {print "set label \"$F[6]\" at $F[0]+.35,$F[6]+'$label_y_offset' font \"arial,15\"\n"}' $in_file_name > label_5.plt
	    ;;
	*)
	    echo $1 bars not supported
	    exit
	    ;;
    esac
}

# tentative, not yet working properly and not used
function load_labels()
{
    num_bars=$1
    headlines=${2:-10}
    taillines=${3:-10}

    create_label_positions $num_bars
    for ((i = 1; i <= $num_bars; i++))
    do
	head -n $headlines label_${i}.plt > head_labels
	tail -n $taillines head_labels >\
            restr_labels_${i}_${headlines}_${taillines}
	echo load \"restr_labels_${i}_${headlines}_${taillines}\" >> tmp.txt
	cat restr_labels_${i}_${headlines}_${taillines}
	rm head_labels
    done
    echo >> tmp.txt
}

function write_basic_plot_conf()
{
    num_bars=$1

    printf "
    set style fill solid 0.8 border -1
    set style data histogram
    set style histogram cluster gap 1
    set mytics
    set xtics scale 0
    set grid y
    set ytics scale 2.0, 1.2
    set bars 3.0
    set boxwidth 1
    set pointsize 4
    set key left samplen 1
    " >> tmp.txt
}

function plot_histograms()
{
    in_file_name=$1
    out_file_name=$2
    x_label=${3:-"No x label!"}
    x_label_offset=${4:-0}
    max_y=$5
    y_label=$6
    num_bars=$7
    plot_curves=$8
    ref_label=$9
    ref_value=${10}

    rm -f tmp.txt
    write_basic_plot_conf $7
    printf "
    set xlabel \"$x_label\" offset 0,$x_label_offset
    set ylabel \"$y_label\"
    set yrange [0:$max_y]
    " >> tmp.txt

    create_label_positions $num_bars
    for ((i = 1; i <= $num_bars; i++))
    do
	echo load \"label_${i}.plt\" >> tmp.txt
    done

    case $mode in
	eps)
	printf "
        set style fill pattern 1
        set output \"${out_file_name}.eps\"
        set term post eps 22
        " >> tmp.txt
	options="-mono"
	    ;;
	gif)
	printf "
        set key horizontal 8000, 30
        set output \"${out_file_name}.gif\"
        set term gif
        " >> tmp.txt
	    ;;
	*)
	printf "
	set term x11 $plot_id font \"arial,12\"
        " >> tmp.txt
	options="-persist"
	    ;;
    esac

    printf "plot " >> tmp.txt
 
    if [ "$ref_label" != "" ] ; then
	printf "%f t \"$ref_label\" lw $lw, " $ref_value >> tmp.txt
    fi

    echo $plot_curves >> tmp.txt
    
    gnuplot $options < tmp.txt
    rm -f label_*.plt tmp.txt

    plot_id=$(($plot_id+1))
}


function plot_4_time_histograms()
{
    y_label="Execution time [ns]"
    in_file_name=$1
    out_file_name=$2
    height=$3
    
    plot_histograms $in_file_name $out_file_name \
	"Flow set" 0 $height "$y_label" 4 \
	"\"$in_file_name\" \
             using 3:xticlabels(2) t \"QFQ+\", \
             \"\" using 4 t \"DRR\", \
             \"\" using 5 t \"QFQ\", \
             \"\" using 6 t \"FIFO\" "
    
    if [ $mode != "x11" ] ; then
    	echo Wrote $out_file_name.$mode
    fi
}



plot_4_time_histograms $dirname/test-time-b0.out $dirname/test-time-b0 600

